//Aerial Perspective Macro written by Nicholas Warren March 2004
//All of the following code is my own work
//This file is to be placed in the shake/include/startup folder

//This macro simulates aerial perspective, the effect of layers becoming bluer
//and less distinct over distance.

//The floowing is the macro definition

image AerialPerspective(
	//the image inputs (creates seven knots on the node)
	image input1,
	image input2,
	image input3,
	image input4,
	image input5,
	image input6,
	image input7,
	//RGB variables for the colour selection
	float red = 0.533333,
	float green = 0.603922,
	float blue = 0.752941,
	float red1 = 0.647059,
	float green1 = 0.666667,
	float blue1 = 0.823529,
	//A set of variables to specify the layers depth in relation to the camera
	float depth1 = 0,
	float depth2 = 0.1,
	float depth3 = 0.2,
	float depth4 = 0.3,
	float depth5 = 0.4,
	float depth6 = 0.5,
	float depth7 = 0.6,
	//A variable to allow for the change in the degree of the calculation
	int degree = 2,
	//A variable to allow the anount of saturation to be changed
	float satValue = 1,
	//a variable to aalow the amount of defocus to be changed
	float defocusValue =1,
	//A variable to allow the selction of the shape of the defocus effect
	string shape = "fast gaussian",
	//A variable to allow for the change in the camera's relation to the sun
	float sunDirection =180,
	//A variable to allow particle density to be increased (bluer, less distinct)
	float particleDensity = 0,
	//A variable to allow cloudDensity to be increased (greyer)
	float cloudDensity = 0,
	//Variables to allow the premultiplication of the layers to be toggled
	int premult1 = 1,
	int premult2 = 1,
	int premult3 = 1,
	int premult4 = 1,
	int premult5 = 1,
	int premult6 = 1,
)
//Macro body

{

//The following two nodes hold the colours that will be used to interpolate the effect.Each of the RGB
//values has a calculation which is the sum of the various components affecting that channel

Color1 = Color(1,1,1,clamp(red+cloudDensity,0,(blue+(particleDensity/0.5))-0.1),
	clamp(green+cloudDensity,0,(blue+(particleDensity/0.5))-0.1),
	clamp((blue+(particleDensity/0.5)),0,1),0,0);
Color2 = Color(1,1,1,clamp(red1+cloudDensity,0,(blue+(particleDensity/0.5))-0.1),
	clamp(green1+cloudDensity,0,(blue+(particleDensity/0.5))-0.1),
	clamp((blue1+(particleDensity/0.5)),0,1),0,0);

//The following nodes allow the saturation of the layers to be altered, they interpolate between complete
//saturation (value of 1) and no saturation (value of 0), the function uses the depth of each layer as 
//the t parameter in the form n1(1-t) + n2t. Each of the depth values is raised to the power of the degree

Saturation1 = Saturation(input1,(1-pow(depth1,degree))+(satValue*pow(depth1,degree)));
Saturation2 = Saturation(input2,(1-pow(depth2,degree))+(satValue*pow(depth2,degree)));
Saturation3 = Saturation(input3,(1-pow(depth3,degree))+(satValue*pow(depth3,degree)));
Saturation4 = Saturation(input4,(1-pow(depth4,degree))+(satValue*pow(depth4,degree)));
Saturation5 = Saturation(input5,(1-pow(depth5,degree))+(satValue*pow(depth5,degree)));
Saturation6 = Saturation(input6,(1-pow(depth6,degree))+(satValue*pow(depth6,degree)));
Saturation7 = Saturation(input7,(1-pow(depth7,degree))+(satValue*pow(depth7,degree)));

//The following nodes compress each layers colour values into an interpolated range. The interpolation is linear
//in the same form as above. The difference being it is interpolated between two interpolated values, thus allowing
//for the effects of the camera looking directly at the sun. It takes the following form:
//     (n1(1-t) + n2t)(1-t) + (n3(1-t) + n4t)t
//Each of the layers depths is plugged in to the calculation to determine that layers compression colours. The depth
//is raised to the power of the degree. Each RGB value is clamped between the range 0 to 1.

Compress1 = Compress(Saturation1, 
	clamp(((0.01*(1-pow(depth1,degree))+(0.6*pow(depth1,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth1,degree))) + (Color1.red*pow(depth1,degree)))),0,1),
	clamp(((0.01*(1-pow(depth1,degree))+(0.6*pow(depth1,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth1,degree))) + (Color1.green*pow(depth1,degree)))),0,1),
	clamp(((0.01*(1-pow(depth1,degree))+(0.6*pow(depth1,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth1,degree))) + (Color1.blue*pow(depth1,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth1,degree))+(1*pow(depth1,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth1,degree)) + (Color2.red*pow(depth1,degree)))),0,1),
	clamp((0.65*(1-pow(depth1,degree))+(1*pow(depth1,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth1,degree)) + (Color2.green*pow(depth1,degree)))),0,1),
	clamp((0.65*(1-pow(depth1,degree))+(1*pow(depth1,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth1,degree)) + (Color2.blue*pow(depth1,degree)))),0,1),
	1);

Compress2 = Compress(Saturation2, 
	clamp(((0.1*(1-pow(depth2,degree))+(0.6*pow(depth2,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth2,degree))) + (Color1.red*pow(depth2,degree)))),0,1),
	clamp(((0.1*(1-pow(depth2,degree))+(0.6*pow(depth2,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth2,degree))) + (Color1.green*pow(depth2,degree)))),0,1),
	clamp(((0.1*(1-pow(depth2,degree))+(0.6*pow(depth2,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth2,degree))) + (Color1.blue*pow(depth2,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth2,degree))+(1*pow(depth2,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth2,degree)) + (Color2.red*pow(depth2,degree)))),0,1),
	clamp((0.65*(1-pow(depth2,degree))+(1*pow(depth2,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth2,degree)) + (Color2.green*pow(depth2,degree)))),0,1),
	clamp((0.65*(1-pow(depth2,degree))+(1*pow(depth2,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth2,degree)) + (Color2.blue*pow(depth2,degree)))),0,1),
	1);

Compress3 = Compress(Saturation3, 
	clamp(((0.01*(1-pow(depth3,degree))+(0.6*pow(depth3,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth3,degree))) + (Color1.red*pow(depth3,degree)))),0,1),
	clamp(((0.01*(1-pow(depth3,degree))+(0.6*pow(depth3,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth3,degree))) + (Color1.green*pow(depth3,degree)))),0,1),
	clamp(((0.01*(1-pow(depth3,degree))+(0.6*pow(depth3,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth3,degree))) + (Color1.blue*pow(depth3,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth3,degree))+(1*pow(depth3,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth3,degree)) + (Color2.red*pow(depth3,degree)))),0,1),
	clamp((0.65*(1-pow(depth3,degree))+(1*pow(depth3,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth3,degree)) + (Color2.green*pow(depth3,degree)))),0,1),
	clamp((0.65*(1-pow(depth3,degree))+(1*pow(depth3,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth3,degree)) + (Color2.blue*pow(depth3,degree)))),0,1),
	1);

Compress4 = Compress(Saturation4, 
	clamp(((0.01*(1-pow(depth4,degree))+(0.6*pow(depth4,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth4,degree))) + (Color1.red*pow(depth4,degree)))),0,1),
	clamp(((0.01*(1-pow(depth4,degree))+(0.6*pow(depth4,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth4,degree))) + (Color1.green*pow(depth4,degree)))),0,1),
	clamp(((0.01*(1-pow(depth4,degree))+(0.6*pow(depth4,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth4,degree))) + (Color1.blue*pow(depth4,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth4,degree))+(1*pow(depth4,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth4,degree)) + (Color2.red*pow(depth4,degree)))),0,1),
	clamp((0.65*(1-pow(depth4,degree))+(1*pow(depth4,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth4,degree)) + (Color2.green*pow(depth4,degree)))),0,1),
	clamp((0.65*(1-pow(depth4,degree))+(1*pow(depth4,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth4,degree)) + (Color2.blue*pow(depth4,degree)))),0,1),
	1);

Compress5 = Compress(Saturation5, 
	clamp(((0.01*(1-pow(depth5,degree))+(0.6*pow(depth5,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth5,degree))) + (Color1.red*pow(depth5,degree)))),0,1),
	clamp(((0.01*(1-pow(depth5,degree))+(0.6*pow(depth5,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth5,degree))) + (Color1.green*pow(depth5,degree)))),0,1),
	clamp(((0.01*(1-pow(depth5,degree))+(0.6*pow(depth5,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth5,degree))) + (Color1.blue*pow(depth5,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth5,degree))+(1*pow(depth5,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth5,degree)) + (Color2.red*pow(depth5,degree)))),0,1),
	clamp((0.65*(1-pow(depth5,degree))+(1*pow(depth5,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth5,degree)) + (Color2.green*pow(depth5,degree)))),0,1),
	clamp((0.65*(1-pow(depth5,degree))+(1*pow(depth5,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth5,degree)) + (Color2.blue*pow(depth5,degree)))),0,1),
	1);

Compress6 = Compress(Saturation6, 
	clamp(((0.01*(1-pow(depth6,degree))+(0.6*pow(depth6,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth6,degree))) + (Color1.red*pow(depth6,degree)))),0,1),
	clamp(((0.01*(1-pow(depth6,degree))+(0.6*pow(depth6,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth6,degree))) + (Color1.green*pow(depth6,degree)))),0,1),
	clamp(((0.01*(1-pow(depth6,degree))+(0.6*pow(depth6,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth6,degree))) + (Color1.blue*pow(depth6,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth6,degree))+(1*pow(depth6,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth6,degree)) + (Color2.red*pow(depth6,degree)))),0,1),
	clamp((0.65*(1-pow(depth6,degree))+(1*pow(depth6,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth6,degree)) + (Color2.green*pow(depth6,degree)))),0,1),
	clamp((0.65*(1-pow(depth6,degree))+(1*pow(depth6,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth6,degree)) + (Color2.blue*pow(depth6,degree)))),0,1),
	1);

Compress7 = Compress(Saturation7, 
	clamp(((0.01*(1-pow(depth7,degree))+(0.6*pow(depth7,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth7,degree))) + (Color1.red*pow(depth7,degree)))),0,1),
	clamp(((0.01*(1-pow(depth7,degree))+(0.6*pow(depth7,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth7,degree))) + (Color1.green*pow(depth7,degree)))),0,1),
	clamp(((0.01*(1-pow(depth7,degree))+(0.6*pow(depth7,degree)))*(1-((sunDirection/180)+cloudDensity))) + 
		(((sunDirection/180)+cloudDensity)*((0.01*(1-pow(depth7,degree))) + (Color1.blue*pow(depth7,degree)))),0,1),
	0,
	clamp((0.65*(1-pow(depth7,degree))+(1*pow(depth7,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth7,degree)) + (Color2.red*pow(depth7,degree)))),0,1),
	clamp((0.65*(1-pow(depth7,degree))+(1*pow(depth7,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth7,degree)) + (Color2.green*pow(depth7,degree)))),0,1),
	clamp((0.65*(1-pow(depth7,degree))+(1*pow(depth7,degree)))*(1-((sunDirection/180)+cloudDensity))+ 
		(((sunDirection/180)+cloudDensity)*((1-pow(depth7,degree)) + (Color2.blue*pow(depth7,degree)))),0,1),
	1);

//The following nodes determine the amount of defocus that each layer gets, by linearly interpolating based upon 
//the layers depth and the focus amount. Each of the depth values is raised to thepower of the degree.

Defocus1 = Defocus(Compress1, 
	(1-pow(depth1,degree))+((defocusValue+(20*particleDensity))*pow(depth1,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);
Defocus2 = Defocus(Compress2, 
	(1-pow(depth2,degree))+((defocusValue+(20*particleDensity))*pow(depth2,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);
Defocus3 = Defocus(Compress3, 
	(1-pow(depth3,degree))+((defocusValue+(20*particleDensity))*pow(depth3,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);
Defocus4 = Defocus(Compress4, 
	(1-pow(depth4,degree))+((defocusValue+(20*particleDensity))*pow(depth4,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);
Defocus5 = Defocus(Compress5, 
	(1-pow(depth5,degree))+((defocusValue+(20*particleDensity))*pow(depth5,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);
Defocus6 = Defocus(Compress6, 
	(1-pow(depth6,degree))+((defocusValue+(20*particleDensity))*pow(depth6,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);
Defocus7 = Defocus(Compress7, 
	(1-pow(depth7,degree))+((defocusValue+(20*particleDensity))*pow(depth7,degree)),
 	xPixels, "rgba",100, shape, 0.95, 1);

//The following nodes combine all of the layers together in their correct order, and takes a variable that allows
//the user to specify whether the layer is to be premultiplied or not.

Over1 = Over(Defocus1, Defocus2, 1, premult1, 0);
Over2 = Over(Over1, Defocus3, 1, premult2, 0);
Over3 = Over(Over2, Defocus4, 1, premult3, 0);
Over4 = Over(Over3, Defocus5, 1, premult4, 0);
Over5 = Over(Over4, Defocus6, 1, premult5, 0);
Over6 = Over(Over5, Defocus7, 1, premult6, 0);

//This statement tells the macro to return the output of thelast node in the chain which is Over6

return Over6;
}